
local canvas = love.graphics.newCanvas(2048,2048)

local map = {[0]={[0]={x=0,y=0,d=0}}}

--[[for i = -14,14 do
	map[i] = {}
	for j = -6,6 do
		map[i][j] = {x=j*32+(i-math.floor(i/2)*2)*24+j*math.cos(math.pi/3)*32,y=i*16*math.sin(math.pi/3),d=0}
	end
end]]

local dir = {
	leftUp = 1,
	up = 2,
	rightUp = 4,
	rightDown = 8,
	down = 16,
	leftDown = 32,
}
local path = {{0,0}}

local surrounds = {}
surrounds[0] = {
	rightDown={x=0,y=1},
	leftDown={x=-1,y=1},
	rightUp={x=0,y=-1},
	leftUp={x=-1,y=-1},
	up={x=0,y=-2},
	down={x=0,y=2},
}
surrounds[1] = {
	leftDown={x=0,y=1},
	rightDown={x=1,y=1},
	leftUp={x=0,y=-1},
	rightUp={x=1,y=-1},
	up={x=0,y=-2},
	down={x=0,y=2},
}

local size = 16

local cMap = {[0]={[0]=true}}
local t = {}

for i = 1, 16 do
	for i=#t, 1, -1 do
		table.remove(t,i)
	end
	for i,v in pairs(cMap) do
		for j,w in pairs(v) do
			table.insert(t,j)
			table.insert(t,i)
		end
	end
	for i = 2, #t, 2 do
		local x,y = t[i-1],t[i]
		for i,v in pairs(surrounds[y-math.floor(y/2)*2]) do
			if not cMap[y+v.y] then cMap[y+v.y] = {} end
			cMap[y+v.y][x+v.x] = true
		end
	end
end

local function checkMap(x,y)
	if cMap[y] then
		return cMap[y][x]
	else
		return nil
	end
end

local function getMap(x,y)
	if map[y] then
		if map[y][x] then
			return map[y][x]
		elseif checkMap(x,y) then
			return 0
		else
			return -1
		end
	elseif checkMap(x,y) then
		return 0
	else
		return -1
	end
end

local function setMap(x,y,v)
	if not map[y] then map[y] = {} end
	map[y][x] = {x=x*32+(y-math.floor(y/2)*2)*24+x*math.cos(math.pi/3)*32,y=y*16*math.sin(math.pi/3),d=v}
end

local function addMap(x,y,v)
	map[y][x].d = map[y][x].d + v
end

local function dirOf(i)
	local lu,u,ru,rd,d,ld = false,false,false,false,false,false
	if i >= dir.leftDown then
		i=math.mod(i,dir.leftDown)
		ld=true
	end
	if i >= dir.down then
		i=math.mod(i,dir.down)
		d=true
	end
	if i >= dir.rightDown then
		i=math.mod(i,dir.rightDown)
		rd=true
	end
	if i >= dir.rightUp then
		i=math.mod(i,dir.rightUp)
		ru=true
	end
	if i >= dir.up then
		i=math.mod(i,dir.up)
		u=true
	end
	if i >= dir.leftUp then
		i=math.mod(i,dir.leftUp)
		lu=true
	end
	return lu,u,ru,rd,d,ld
end

local function dirsOf(i)
	local t = {false,false,false,false,false,false}
	for j = 6,1,-1 do
		if i >= (2^j)/2 then
			t[j] = true
			i = i - (2^j)/2
		end
	end
	return t
end

function love.load()

end

function drawMapFrom(x,y,d)
	drawMap({x=x*32+(y-math.floor(y/2)*2)*24+x*math.cos(math.pi/3)*32,y=y*16*math.sin(math.pi/3),d=d},true)
end

function drawMap(map,color)
	local lu,u,ru,rd,d,ld = dirOf(map.d) --false,false,false,false,false,false
	local x,y = map.x,map.y
	if not color then
		love.graphics.setColor(63,63,127)
	end
	love.graphics.polygon('fill',
		x-math.cos(math.pi/3)*size,y-math.sin(math.pi/3)*size,
		x-math.cos(math.pi/3*2)*size,y-math.sin(math.pi/3*2)*size,
		x-math.cos(math.pi)*size,y-math.sin(math.pi)*size,
		x-math.cos(math.pi/3*4)*size,y-math.sin(math.pi/3*4)*size,
		x-math.cos(math.pi/3*5)*size,y-math.sin(math.pi/3*5)*size,
		x-math.cos(math.pi*2)*size,y-math.sin(math.pi*2)*size)
	love.graphics.setColor(255,255,255)
	if not lu then
		love.graphics.line(x-size,y,x-math.cos(math.pi/3)*size,y-math.sin(math.pi/3)*size)
	end
	if not u then
		love.graphics.line(x-math.cos(math.pi/3)*size,y-math.sin(math.pi/3)*size,x+math.cos(math.pi/3)*size,y-math.sin(math.pi/3)*size)
	end
	if not ru then
		love.graphics.line(x+size,y,x+math.cos(math.pi/3)*size,y-math.sin(math.pi/3)*size)
	end
	if not rd then
		love.graphics.line(x+size,y,x+math.cos(math.pi/3)*size,y+math.sin(math.pi/3)*size)
	end
	if not d then
		love.graphics.line(x-math.cos(math.pi/3)*size,y+math.sin(math.pi/3)*size,x+math.cos(math.pi/3)*size,y+math.sin(math.pi/3)*size)
	end
	if not ld then
		love.graphics.line(x-size,y,x-math.cos(math.pi/3)*size,y+math.sin(math.pi/3)*size)
	end
end

function updateTiles()
	love.graphics.setColor(255,255,255)
	canvas:clear()
	love.graphics.setCanvas(canvas)
	love.graphics.translate(1024,1024)
	for i,v in pairs(map) do
		for j,w in pairs(v) do
			drawMap(w)
		end
	end
	love.graphics.setCanvas()
end

local function reverse(i)
	if i > 4 then
		return i / 8
	else
		return i * 8
	end
end

function updateMap(map)
	local x,y = path[#path][1],path[#path][2]
	local goTo = {}
	for i,v in pairs(surrounds[y-math.floor(y/2)*2]) do
		if getMap(x+v.x,y+v.y) == 0 then
			table.insert(goTo,i)
		end
	end
	if #goTo > 0 then
		local c = math.floor(#goTo * math.random())+1
		setMap(x+surrounds[y-math.floor(y/2)*2][goTo[c]].x,y+surrounds[y-math.floor(y/2)*2][goTo[c]].y,reverse(dir[goTo[c]]))
		addMap(x,y,dir[goTo[c]])
		table.insert(path,{x+surrounds[y-math.floor(y/2)*2][goTo[c]].x,y+surrounds[y-math.floor(y/2)*2][goTo[c]].y})
	else
		path[#path] = nil
	end
end

local drawFromX,drawFromY = 0,0

function love.keypressed(k)
	if k == ' ' then
		updateTiles()
	end
end

local mouseX,mouseY,mouseOldX,mouseOldY = 0,0,0,0

function love.update(dt)
	mouseX,mouseY=love.mouse.getPosition()
	if #path > 0 then
		updateMap(map)
		updateTiles()
	end
	if love.keyboard.isDown 'up' then
		drawFromY=drawFromY-120*dt
	end
	if love.keyboard.isDown 'down' then
		drawFromY=drawFromY+120*dt
	end
	if love.keyboard.isDown 'left' then
		drawFromX=drawFromX-120*dt
	end
	if love.keyboard.isDown 'right' then
		drawFromX=drawFromX+120*dt
	end
	if love.mouse.isDown "l" then
		drawFromX=drawFromX+mouseOldX-mouseX
		drawFromY=drawFromY+mouseOldY-mouseY
	end
	mouseOldX,mouseOldY=mouseX,mouseY
end

function love.draw()
	love.graphics.setLineWidth(2)
	love.graphics.push()
	love.graphics.setColor(255,255,255)
	love.graphics.translate(400-drawFromX,300-drawFromY)
	love.graphics.draw(canvas,-1024,-1024)
	if #path > 0 then
		for i = #path, math.max(1,#path-1), -1 do
			love.graphics.setColor(255,0,0)
			drawMapFrom(path[i][1],path[i][2],map[path[i][2]][path[i][1]].d)
		end
	end
	--[[size=14
	drawMapFrom(drawFromX,drawFromY,0)
	size=16
	love.graphics.setColor(0,255,0)
	for i,v in pairs(surrounds[drawFromY-math.floor(drawFromY/2)*2]) do
		drawMapFrom(drawFromX+v.x,drawFromY+v.y,0)
	end]]
	love.graphics.pop()
end
